﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace DxWatcher
{
    public partial class BandMapDlg : Form
    {
        public string band = ""; // selected band
        List<spot> spots = new List<spot>();
        TimeSpan timeout = new TimeSpan(1, 0, 0);   // spot life time
        TimeSpan freshtime = new TimeSpan(0, 10, 0); // fresh spot time
        MainForm parent;
        public List<mapSpot> mapSpots=  new List<mapSpot>();
        Point lastMouse;
        Point lastPointInfoMouse;
        SpotInfoDlg spotInfo;


        public BandMapDlg(MainForm parent)
        {
            this.parent = parent;
            InitializeComponent();
            spotInfo = new SpotInfoDlg(parent);
            dxScale1.map = this;
        }

        private void BandMap_FormClosing(object sender, FormClosingEventArgs e)
        {
            this.Hide();
            e.Cancel = true;
        }

        private void BandMap_SizeChanged(object sender, EventArgs e)
        {
            dxScale1.SetScale();
            Invalidate();
        }

        private void BandMap_Shown(object sender, EventArgs e)
        {
            cbBand.Items.Clear();
            foreach (DxBand bandx in parent.bands)
            {
                cbBand.Items.Add(bandx);
                if (band == bandx.name)
                    cbBand.SelectedItem = bandx;
            }
            if ((cbBand.SelectedIndex < 0) && (parent.bands.Count > 0))
                cbBand.SelectedIndex = 0;
        }

        private void butVfo_FollowBand(object sender, EventArgs e)
        {
            double vfo = sender == this.butVFOA ? parent.tcvr.vfoA : parent.tcvr.vfoB;
            foreach (object cbItem in cbBand.Items)
            {
                if (cbItem is DxBand)
                {
                    DxBand band = cbItem as DxBand;
                    if ((vfo >= band.freq1) && (vfo <= band.freq2))
                    {
                        cbBand.SelectedItem = cbItem;
                        return;
                    }
                }
            }
        }

        private void cbBand_SelectedIndexChanged(object sender, EventArgs e)
        {
            dxScale1.Focus();
            DxBand selectedBand = cbBand.SelectedItem as DxBand;
            band = selectedBand.name;
            parent.SaveBandMapSetting();
            dxScale1.FreqStart = selectedBand.freq1;
            dxScale1.FreqWindow = selectedBand.freq2 - selectedBand.freq1;
            spots.Clear();
            parent.UpdateBandMapSpots();
            dxScale1.SetScale();
        }

        /// <summary>
        /// Add spot from current band that is not too old to bandmap
        /// return TRUE when spot accepted
        /// </summary>
        /// <param name="s"></param>
        public bool NewSpot(spot s)
        {

            TimeSpan span = DateTime.Now.ToUniversalTime() - s.utc;
            if ((s.band == band) && (span < timeout))
            {
                bool done = false;
                foreach (spot s2 in spots)
                {
                    if (s2.call == s.call)
                    {
                        if (s.utc > s2.utc) //  remove obsolete spot
                        {
                            spots.Remove(s2);
                            break;
                        } 
                        done = true;    // callsign in table exists with newer timestamp
                    }
                }
                if (!done)
                {
                    spots.Add(s);   // callsing not yet in table, add it 
                    spots.Sort(SpotFreqComparer);
                    return true;
                }
            }
            return false;
        }

        /// <summary>
        /// Remove obsolete spots. Called every 1 minute
        /// returns true, when any spot removed
        /// </summary>
        public bool CleanupSpots()
        {
            bool done = false;
            bool doRefresh = false;
            while (!done)
            {
                done = true;
                foreach (spot s in spots)
                {
                    if ((DateTime.Now.ToUniversalTime() - s.utc) > timeout)
                    {
                        spots.Remove(s);
                        done = false;
                        doRefresh = true;
                        break;
                    }
                }
            }
            return doRefresh;
        }

        /// <summary>
        /// Comparer to sort spots by frequency
        /// </summary>
        /// <param name="a"></param>
        /// <param name="b"></param>
        /// <returns></returns>
        private int SpotFreqComparer(spot a, spot b)
        {
            if (a.mhz > b.mhz)
                return 1;
            if (a.mhz < b.mhz)
                return -1;
            return 0;
        }

        private void BandMap_Paint(object sender, PaintEventArgs e)
        {
            //if (!dxScale1.done)
            //    return;
            Font fnt1 = new Font(FontFamily.GenericSansSerif,8.0f);
            Font fnt2 = new Font(FontFamily.GenericSansSerif, 8.0f, FontStyle.Bold);
            int lastPosn = 0;
            mapSpots.Clear();
            foreach (spot s in spots)
            {

                if ((s.mhz >= dxScale1.FreqStart) && (s.mhz <= (dxScale1.FreqStart + dxScale1.FreqWindow)))
                {
                    int posn = (int)((s.mhz - dxScale1.FreqStart) / dxScale1.FreqWindow * dxScale1.Height) + dxScale1.Top;
                    int posn2 = Math.Max(posn, lastPosn + 15);
                    lastPosn = posn2;
                    Brush br = Brushes.Black;
                    switch (s.cont)
                    {
                        case "OC":
                            br=Brushes.Red;
                            break;
                        case "AF":
                            br = Brushes.Green;
                            break;
                        case "NA":
                            br = Brushes.Blue;
                            break;
                        case "SA":
                            br = Brushes.DarkOrange;
                            break;
                        case "AS":
                            br = Brushes.DarkViolet;
                            break;
                    }
                    Font fnt = (DateTime.Now.ToUniversalTime() - s.utc) <= freshtime ? fnt2 : fnt1;
                    e.Graphics.DrawString(s.call, fnt, br , 10.0f, (float)(posn2 - 7));
                    SizeF ssize = e.Graphics.MeasureString(s.call, fnt);
                    e.Graphics.DrawLine(Pens.Black, 12.0f + ssize.Width, posn2, dxScale1.Left, posn);
                    mapSpot ms = new mapSpot();
                    ms.rect = new RectangleF(new PointF(10.0f, posn2 - 7), ssize);
                    ms.call = s.call;
                    mapSpots.Add(ms);
                }
            }

            double vfo = parent.tcvr.vfoB;
            if ((vfo >= dxScale1.FreqStart) && (vfo <= (dxScale1.FreqStart + dxScale1.FreqWindow)))
            {
                int posn = (int)((vfo - dxScale1.FreqStart) / dxScale1.FreqWindow * dxScale1.Height) + dxScale1.Top;
                Point[] pts = new Point[3];
                pts[0] = new Point(dxScale1.Left, posn);
                pts[1] = new Point(dxScale1.Left - 7, posn - 5);
                pts[2] = new Point(dxScale1.Left - 7, posn + 5);
                e.Graphics.FillPolygon(Brushes.Green, pts);
            }

            vfo = parent.tcvr.vfoA;
            if ((vfo >= dxScale1.FreqStart) && (vfo <= (dxScale1.FreqStart + dxScale1.FreqWindow)))
            {
                int posn = (int)((vfo - dxScale1.FreqStart) / dxScale1.FreqWindow * dxScale1.Height) + dxScale1.Top;
                Point[] pts = new Point[3];
                pts[0] = new Point(dxScale1.Left, posn);
                pts[1] = new Point(dxScale1.Left-7, posn-5);
                pts[2] = new Point(dxScale1.Left-7, posn+5);
                e.Graphics.FillPolygon(Brushes.Red, pts);
            }
        }

        /// <summary>
        /// Structure holding screen coordinates of spot callsign
        /// </summary>
        public struct mapSpot
        {
            public RectangleF rect;
            public string call;
        }

        private void BandMap_MouseMove(object sender, MouseEventArgs e)
        {
            if (spotInfo.active)
            {
                spotInfo.active = false;
                spotInfo.Hide();
                // BringToFront();
            }
            lastMouse = e.Location;
            timer1.Stop();
            timer1.Interval = 500;
            timer1.Start();
        }

        private void timer1_Tick(object sender, EventArgs e)
        {
            if ((spotInfo.active == false) && (lastMouse != lastPointInfoMouse))
            {
                foreach (mapSpot ms in mapSpots)
                {
                    if ((ms.rect.Left <= lastMouse.X) && (ms.rect.Right >= lastMouse.X) && (ms.rect.Top <= lastMouse.Y) && (ms.rect.Bottom >= lastMouse.Y))
                    {
                        spotInfo.lblCall.Text = ms.call;
                        spotInfo.lblFreq.Text = "??? MHz";
                        spotInfo.lblBy.Text = "by ???";
                        spotInfo.lblUtc.Text = "at ??:??Z";
                        foreach (spot s in spots)
                        {
                            if (s.call == ms.call)
                            {
                                spotInfo.freq = s.mhz;
                                spotInfo.lblFreq.Text = s.mhz.ToString("F04") + " MHz";
                                spotInfo.lblBy.Text = "by " + s.spotter;
                                spotInfo.lblUtc.Text = "at " + s.utc.ToString("HH:mm") + "Z";
                                break;
                            }
                        }
                        spotInfo.Location = new Point(lastMouse.X + Left, lastMouse.Y + Top - 35);
                        lastPointInfoMouse = lastMouse;
                        spotInfo.active = true;
                        spotInfo.Show();
                        spotInfo.BringToFront();
                        timer1.Stop();
                        timer1.Interval = 2000; // display time
                        timer1.Start();
                        break;
                    }
                }
            }
            else if (spotInfo.active)
            {
                spotInfo.active = false;
                spotInfo.Hide();
                // BringToFront();
            }
        }

        private void BandMapDlg_MouseClick(object sender, MouseEventArgs e)
        {
            foreach (mapSpot ms in mapSpots)
            {
                if ((ms.rect.Left <= lastMouse.X) && (ms.rect.Right >= lastMouse.X) && (ms.rect.Top <= lastMouse.Y) && (ms.rect.Bottom >= lastMouse.Y))
                {
                    foreach (spot s in spots)
                    {
                        if (s.call == ms.call)
                        {
                            parent.tcvr.vfoA = s.mhz;
                        }
                    }
                }
            }
        }

    }
}
