﻿using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Windows.Forms;

namespace DxWatcher
{
    /// <summary>
    /// Reads ctry.dat, creates country table
    /// offers identification of callsign (country, continent,..)
    /// </summary>
    class DxCountry
    {

        // one country in country list
        public struct country
        {
            public string pfx;
            public string name;
            public string cont;

            public override string ToString()
            {
                return pfx + " - " + name;
            }
        }

        private class entry:IComparer<entry>
        {
            public int idx; // referring to country
            public string data; // part of callsign

            public int Compare(entry x, entry y)
            {
                return x.data.CompareTo(y.data);
            }

            public override string ToString()
            {
                return data;
            }
        }

        public List<country> countries = new List<country>();
        List<List<entry>> tables = new List<List<entry>>();
        static public DxCountry inst;

        public DxCountry()
        {
            inst = this; // one instance globally available
        }

        public bool LoadTables(string filename)
        {
            TextReader reader = new StreamReader(filename);
            string line;
            int index = -1;
            while ((line = reader.ReadLine()) != null)
            {
                if (line.StartsWith("&"))   // continue line
                {
                    line = line.Substring(1).Trim();
                    if (index >= 0)
                        LoadCalls(index, line);
                }
                else
                {
                    int psn = line.IndexOf(':');
                    if (psn < 1)
                        continue;
                    string name = line.Substring(0, psn);   // country name
                    line = line.Substring(psn + 1).Trim();

                    if (line.StartsWith("*"))
                        line = line.Substring(1).Trim();

                    psn = line.IndexOf(':');
                    if (psn < 1)
                        continue;
                    string ps = line.Substring(0, psn);
                    int itu = int.Parse(ps);    // ITU zone
                    line = line.Substring(psn + 1).Trim();
                    psn = line.IndexOf(':');

                    if (psn < 1)
                        continue;
                    ps = line.Substring(0, psn);
                    int waz = int.Parse(ps);    // WAZ zone
                    line = line.Substring(psn + 1).Trim();

                    psn = line.IndexOf(':');
                    if (psn < 1)
                        continue;
                    string cont = line.Substring(0, psn);   // continent
                    line = line.Substring(psn + 1).Trim();

                    psn = line.IndexOf(' ');
                    ps = line.Substring(0, psn);    // coordinates, ignored
                    line = line.Substring(psn + 1).Trim();

                    psn = line.IndexOf(':');
                    if (psn < 1)
                        continue;
                    string pfx = line.Substring(0, psn);   // official prefix
                    line = line.Substring(psn + 1).Trim();

                    if (line.Length < 0)
                        continue;   // no prefixes defined

                    country ctry = new country();
                    ctry.name = name;
                    ctry.pfx = pfx;
                    ctry.cont = cont;

                    index = countries.Count;
                    countries.Add(ctry);
                    LoadCalls(index, line);
                }
            }
            foreach (List<entry> table in tables)
            {
                table.Sort(new entry());
            }
            return true;
        }

        private bool LoadCalls(int index, string line)
        {
            int posn = -1;
            string pfx;
            while ((posn = line.IndexOf(',')) > 0)
            {
                pfx = line.Substring(0, posn);
                line = line.Substring(posn + 1);
                AddCall(index, pfx);
            }
            posn = line.IndexOf(';');
            pfx = line.Substring(0, posn);
            AddCall(index, pfx);
            return true;
        }

        private void AddCall(int index, string pfx)
        {
            int sel = pfx.Length;   // table selector based on callsign length
            while (sel > tables.Count)
                tables.Add(new List<entry>());
            List<entry> table = tables[sel - 1];
            entry en = new entry();
            en.idx = index;
            en.data = pfx;
            table.Add(en);
        }


        /// <summary>
        /// Search the prefix tables for match (identify country)
        /// </summary>
        /// <param name="call">string being analyzed</param>
        /// <param name="ctry">matching criteria country</param>
        /// <returns>-1 when not found, >0 when match found - higher number means better match</returns>
        private int Search(string call, ref country ctry)
        {
            try
            {
                int countryidx = -1;
                int len = call.Length;
                if (len > (tables.Count))
                    len = tables.Count;
                while ((countryidx < 0) && (len > 0))
                {
                    for (; len > 0; len--)
                    {
                        List<entry> table = tables[len - 1];
                        if (table.Count > 0)
                        {
                            entry en = new entry();
                            en.data = call.Substring(0, len);
                            int idx = table.BinarySearch(en, en);
                            if (idx >= 0)
                            {
                                countryidx = table[idx].idx;
                                break;
                            }
                        }
                    }
                }
                if (countryidx >= 0)
                {
                    ctry = countries[countryidx];
                }
                return len;
            }
            catch(Exception ex)
            {
                MessageBox.Show(ex.Message + "\r\nCall: " + call, "Function Search");
                return -1;
            }
        }

        public bool Identify(string call, ref country ctry)
        {
            call = call.ToUpper();
            
            int posn = call.LastIndexOf("/MM"); // remove /MM and /AM
            if (posn < 0)
                posn = call.LastIndexOf("/AM");
            if ((posn >= 0) && (posn == call.Length - 3))
                call = call.Substring(0, posn);
            else
            {
                posn = call.LastIndexOf("/QRP");
                if ((posn >= 0) && (posn == call.Length - 4))
                    call = call.Substring(0, posn); // remove /QRP
            }

            posn =call.LastIndexOf('/');
            if ((posn > 0) && (posn == call.Length - 2))   // only one character after "/"
            {
                char nr = call[posn+1];
                call = call.Substring(0, posn); // remove tail
                if ((nr >= '0') && (nr <= '9'))
                {
                    posn = call.LastIndexOfAny(new char[] { '0', '1', '2', '3', '4', '5', '6', '7', '8', '9' });
                    if (posn > 0)
                    {
                        call = call.Remove(posn, 1);
                        string nrs = "" + nr;
                        call = call.Insert(posn, nrs);    // replace number in callsign by number after "/"
                    }
                }
            }
            else if (posn > 0)  // calls like HK0/AL4Q or SM3ABC/SV9
            {
 /*               string callx = call.Substring(0, posn); // try text before "/"
                country ctry1 = new country();
                int result1 = Search(callx, ref ctry1);
                callx = call.Substring(posn + 1);  // and after "/"
                country ctry2 = new country();
                int result2 = Search(callx, ref ctry2);

                ctry = result1 > result2 ? ctry1 : ctry2;  // take that with better match
                return Math.Max(result1, result2) >= 0; */
                string call1 = call.Substring(0, posn); // try text before "/"
                string call2 = call.Substring(posn + 1);  // and after "/"
                call = call2.Length < call1.Length ? call2 : call1;    // analyze always shorter part of callsign      
            }
            return Search(call, ref ctry) >= 0;
        }
    }
}
