﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Text.RegularExpressions;
using System.Xml;
using System.Windows.Forms;

namespace DxWatcher
{
    /// <summary>
    ///  Handling of all spot filtering for filtered spot grid
    /// </summary>
    class SpotFilter
    {
        public enum filterType { any, call, country, continent };

        public class FilterItem
        {
            const string any = "Any";

            public string dx; // can be regexpress, country pfx or continent, selected by callFType
            public filterType dxFType;
            public string band;
            public string mode;
            public string spotter; // can be regexpress, country pfx or continent
            public filterType spotterFType;
            public bool alarm;

            /// <summary>
            /// constructor
            /// </summary>
            /// <param name="dx"></param>
            /// <param name="dxt"></param>
            /// <param name="band"></param>
            /// <param name="mode"></param>
            /// <param name="s"></param>
            /// <param name="st"></param>
            /// <param name="alarm"></param>
            public FilterItem(string dx, filterType dxt, string band, string mode, string s, filterType st, bool alarm)
            {
                this.dx = dx;
                this.dxFType = dxt;
                this.band = band;
                this.mode = mode;
                this.spotter = s;
                this.spotterFType = st;
                this.alarm = alarm;
            }

            public FilterItem()
            {
                this.dx = "";   // DX filter criteria parameter
                this.dxFType = filterType.call;
                this.band = any;
                this.mode = any;
                this.spotter = "";   // spotter filter criteria parameter
                this.spotterFType = filterType.any;
                this.alarm = false;
            }


            /// <summary>
            /// Test whether the spot matches the filter
            /// </summary>
            /// <param name="s">sport</param>
            /// <param name="dxc">country of DX stn</param>
            /// <param name="spc">country of spotter</param>
            /// <param name="alarmActive">alarm flag</param>
            /// <returns>true if matches the filter</returns>
            internal bool TestRule(spot s, DxCountry.country dxc, DxCountry.country spc, ref bool alarmActive)
            {
                if (!TestStation(s.call, dxc, dx, dxFType))
                    return false;

                if ((band != null) && (band != any) && (band.Length > 0) && (s.band != band))
                    return false;

                if ((mode != null) && (mode != any) && (mode.Length > 0) && (mode != s.mode.ToUpper()))
                    return false;

                if (!TestStation(s.spotter, spc, spotter, spotterFType))
                    return false;

                if (alarm)
                    alarmActive = true;
                return true;
            }


            /// <summary>
            /// Test station (dx or spotter) against the rule
            /// </summary>
            /// <param name="stn">call of stn</param>
            /// <param name="ctry">country of stn</param>
            /// <param name="rule">filter rule</param>
            /// <param name="type">filter type</param>
            /// <returns></returns>
            private bool TestStation(string stn, DxCountry.country ctry, string rule, filterType type)
            {
                bool retval = true; // default for "any"
                switch (type)
                {
                    case filterType.call:
                        Regex re = new Regex(rule); // regular expression to callsign
                        retval = re.IsMatch(stn);
                        break;
                    case filterType.country:
                        retval = ctry.pfx == rule;
                        break;
                    case filterType.continent:
                        retval = ctry.cont == rule;
                        break;
                }
                return retval;// any, always accepted
            }
        }


        public List<FilterItem> filters = new List<FilterItem>();
        public static SpotFilter inst;  // only one instance globally accessible


        public SpotFilter()
        {
            inst = this;
        }

        /// <summary>
        /// Take filters one by one, if any matches
        /// </summary>
        /// <param name="s"></param>
        /// <param name="alarmActive"></param>
        /// <returns></returns>

        public bool TestMatch(spot s, ref bool alarmActive, out string dxCont)
        {
            bool retval = false;
            dxCont = "";

            if (filters.Count <= 0)
                return true;        // no filters, all matches

            DxCountry.country dxc = new DxCountry.country();    // DX station / country
            DxCountry.inst.Identify(s.call, ref dxc);
            dxCont = dxc.cont;

            DxCountry.country src = new DxCountry.country();    // spotter / country
            DxCountry.inst.Identify(s.spotter, ref src);

            foreach (FilterItem filter in filters)    // filters can overlap so we need to test all (reason alarm)
            {
                if (filter.TestRule(s, dxc, src, ref alarmActive))
                {
                    retval = true;    // one filter matches, accepted.
                    if (alarmActive)    // no reason to test remaining filters when alarm already active
                        break;
                }
            }
            return retval; // non of filters matches, refused
        }

        public void ShowFilterDialog()
        {

        }

        /// <summary>
        /// Save filter setup to XML file
        /// </summary>
        /// <param name="filename"></param>
        public void SaveFilters(string filename)
        {
            XmlDocument doc = new XmlDocument();
            XmlNode root = doc.CreateElement("Filters");
            doc.AppendChild(root);
            foreach (SpotFilter.FilterItem f in filters)
            {
                XmlNode fnode = doc.CreateElement("Filter");
                root.AppendChild(fnode);

                XmlNode pnode = doc.CreateElement("DX");
                fnode.AppendChild(pnode);

                XmlAttribute val = doc.CreateAttribute("type");
                val.Value = f.dxFType.ToString();
                pnode.Attributes.Append(val);

                val = doc.CreateAttribute("par");
                val.Value = f.dx;
                pnode.Attributes.Append(val);

                pnode = doc.CreateElement("Slot");
                fnode.AppendChild(pnode);

                val = doc.CreateAttribute("band");
                val.Value = f.band;
                pnode.Attributes.Append(val);

                val = doc.CreateAttribute("mode");
                val.Value = f.mode;
                pnode.Attributes.Append(val);

                val = doc.CreateAttribute("alarm");
                val.Value = f.alarm ? "1" : "0";
                pnode.Attributes.Append(val);

                pnode = doc.CreateElement("Spotter");
                fnode.AppendChild(pnode);

                val = doc.CreateAttribute("type");
                val.Value = f.spotterFType.ToString();
                pnode.Attributes.Append(val);

                val = doc.CreateAttribute("par");
                val.Value = f.spotter;
                pnode.Attributes.Append(val);

            }
            doc.Save(filename);

        }

        /// <summary>
        /// Load filter setup from XML file
        /// </summary>
        /// <param name="filename"></param>
        public void LoadFilters(string filename)
        {
            filters.Clear();
            try
            {
                XmlDocument doc = new XmlDocument();
                doc.Load(filename);
                XmlNode root = doc.FirstChild;
                if (root.Name == "Filters")
                {
                    XmlNode f = root.FirstChild;
                    while (f != null)
                    {
                        if (f.Name == "Filter")
                        {
                            FilterItem sf = new FilterItem();
                            XmlNode x = f.FirstChild;
                            while (x != null)
                            {
                                switch (x.Name)
                                {
                                    case "DX":
                                        foreach (XmlAttribute atr in x.Attributes)
                                        {
                                            if (atr.Name == "type")
                                            {
                                                switch (atr.Value)
                                                {
                                                    case "any":
                                                        sf.dxFType = filterType.any;
                                                        break;
                                                    case "call":
                                                        sf.dxFType = filterType.call;
                                                        break;
                                                    case "country":
                                                        sf.dxFType = filterType.country;
                                                        break;
                                                    case "continent":
                                                        sf.dxFType = filterType.continent;
                                                        break;
                                                }
                                            }
                                            else if (atr.Name == "par")
                                            {
                                                sf.dx = atr.Value;
                                            }
                                        }
                                        break;
                                    case "Slot":
                                        foreach (XmlAttribute atr in x.Attributes)
                                        {
                                            if (atr.Name == "band")
                                            {
                                                sf.band = atr.Value;
                                            }
                                            else if (atr.Name == "mode")
                                            {
                                                sf.mode = atr.Value;
                                            }
                                            else if (atr.Name == "alarm")
                                            {
                                                sf.alarm = atr.Value == "1";
                                            }
                                        }
                                        break;
                                    case "Spotter":
                                        foreach (XmlAttribute atr in x.Attributes)
                                        {
                                            if (atr.Name == "type")
                                            {
                                                switch (atr.Value)
                                                {
                                                    case "any":
                                                        sf.spotterFType = filterType.any;
                                                        break;
                                                    case "call":
                                                        sf.spotterFType = filterType.call;
                                                        break;
                                                    case "country":
                                                        sf.spotterFType = filterType.country;
                                                        break;
                                                    case "continent":
                                                        sf.spotterFType = filterType.continent;
                                                        break;
                                                }
                                            }
                                            else if (atr.Name == "par")
                                            {
                                                sf.spotter = atr.Value;
                                            }
                                        }
                                        break;
                                }
                                x = x.NextSibling;
                            }
                            filters.Add(sf);
                        }
                        f = f.NextSibling;
                    }
                }
            }
            catch { MessageBox.Show("Error loading filter setup"); }
        }
    }
}
